@@ -12,21 +12,18 @@
 
 #define SYSLOG_FACILITY	LOG_USER
 
-
-#define VERSION "2.2"
+#define VERSION		"2.3"
 
 #ifndef REMAILER_DIR
 #define REMAILER_DIR	""
 #endif /* !REMAILER_DIR */
 
 #define REMAILER_BIN	"remailer"
-#define MIX_MAIL	"mixmaster -Q"
-#define MIX_NEWS	"mixmaster -Q post:"
 
 /* updatestats() argument values */
 enum stats_flag {
   STATS_MESSAGE,
-  STATS_PGP
+  STATS_PGP,
 };
 
 /* scan_message() return values */
@@ -35,6 +32,8 @@ enum message_type {
   BLOCKED_SENDER,
   NULL_MESSAGE,
   PGP_MESSAGE,
+  REMIX_MESSAGE,
+  REPGP_MESSAGE,
   ANON_MESSAGE,
   ANON_POST,
   STATS_REQ,
@@ -45,13 +44,14 @@ enum message_type {
   ENCRYPT_SUB,
   ENCRYPT_KEY,
   LATENT_TIME,
-  URL_REQUEST,
+  URL_REQUEST
 };
 
 extern int USE_SYSLOG;
 extern int USE_STATS;
 extern int USE_MIX;
 extern int ALLOW_PGP;
+extern int ALLOW_GPG;
 extern int ALLOW_POST;
 extern int ALLOW_WWW;
 extern int ALLOW_LTIME;
@@ -61,18 +61,21 @@ extern char cur_dir[];
 extern char MIX_DIR[];
 extern char QUEUE_DIR[];
 extern char PGP_DIR[];
+extern char GPG_DIR[];
 extern char MAIL_SPOOL[];
+extern char MIXMASTER[];
 extern char SENDMAIL[];
 extern char INEWS[];
 extern char PGP[];
+extern char GPG[];
 extern char WGET[];
 extern char HELP_FILE[];
 extern char KEY_FILE[];
 extern char STATS_FILE[];
 extern char PASS_FILE[];
+extern char TYPE2_LIST[];
 extern char SOURCE_BLOCK[];
 extern char DEST_BLOCK[];
-extern char SUBJ_BLOCK[];
 extern char GROUP_BLOCK[];
 extern char URL_BLOCK[];
 extern char HDRFILTER[];
@@ -86,25 +89,25 @@ extern char COMPLAINTS[];
 
 /* function prototypes */
 int init_remailer(void);
-enum message_type scan_message(const char *, char *, char *, const int);
-void remove_headers(const char *, const char *, const char *);
+enum message_type scan_message(const char *, char *, const int);
+void get_from(const char *, char *);
+void remove_headers(FILE *, FILE *, enum message_type);
 int rxmatch(const char *, const char *);
+void flushleft(char *, const unsigned);
 void chop(char *);
 int strileft(const char *, const char *);
 int strifind(const char *, const char *);
 int strieq(const char *, const char *);
-int blocked(const char *, const char *);
-int print_blocked(FILE *, const char *);
-void cut(const char *, const char *, const char *, const char *, const int);
+int rxfind(const char *, const char *);
+int findline(const char *, const char *);
+int print_list(FILE *, const char *);
+void cut(const char *, const char *, const char *, const char *);
 void mailfile(const char *, const char *);
-const char *basename(const char *);
 void runlatentqueue(void);
 void queuelatent(const char *, const char *);
 int parsetime(const char *, const int);
-int pgpdecrypt(const char *, const char *);
-int encrypt_key(const char *, const char *, const char *);
+int run_cmd(const char *, const char *, const char *, FILE *, FILE *);
 int append_url(const char *, const char *, const char *);
-int scan_subject(const char *);
 void encrypt_sub(const char *, const char *);
 void updatestats(enum stats_flag);
 void mailstats(const char *);
