@@ -22,6 +22,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <fcntl.h>
 #include <time.h>
 #include <syslog.h>
@@ -33,29 +34,33 @@ int USE_SYSLOG = 0;
 int USE_STATS = 1;
 int USE_MIX = 0;
 int ALLOW_PGP = 1;
+int ALLOW_GPG = 0;
 int ALLOW_POST = 1;
 int ALLOW_WWW = 1;
 int ALLOW_LTIME = 1;
 int SIZE_LIMIT = 0;
 char remailer_dir[BUFSIZ] = "";
 char cur_dir[BUFSIZ];
+char TYPE2_LIST[BUFSIZ] = "";
 char MIX_DIR[BUFSIZ] = "";
 char QUEUE_DIR[BUFSIZ] = "";
 char PGP_DIR[BUFSIZ] = "";
+char GPG_DIR[BUFSIZ] = "";
 char MAIL_SPOOL[BUFSIZ] = "/dev/null";
+char MIXMASTER[BUFSIZ] = "";
 /* the -t option tells sendmail to expect a To: header in stdin */
 char SENDMAIL[BUFSIZ] = "/usr/sbin/sendmail -t";
 /* the -h option tells inews to expect message headers in stdin */
 char INEWS[BUFSIZ] = "/usr/bin/inews -h";
 char PGP[BUFSIZ] = "/usr/bin/pgp";
+char GPG[BUFSIZ] = "/usr/local/bin/gpg";
 char WGET[BUFSIZ] = "/usr/bin/wget";
 char HELP_FILE[BUFSIZ] = "freedom-help";
 char STATS_FILE[BUFSIZ] = "freedom-stats";
 char KEY_FILE[BUFSIZ] = "freedom-key";
-char PASS_FILE[BUFSIZ] = "";
+char PASS_FILE[BUFSIZ] = "passphrase";
 char SOURCE_BLOCK[BUFSIZ] = "blocked.source";
 char DEST_BLOCK[BUFSIZ] = "blocked.destination";
-char SUBJ_BLOCK[BUFSIZ] = "blocked.subject";
 char GROUP_BLOCK[BUFSIZ] = "blocked.newsgroup";
 char URL_BLOCK[BUFSIZ] = "blocked.url";
 char HDRFILTER[BUFSIZ] = "headers.del";
@@ -69,12 +74,12 @@ char COMPLAINTS[BUFSIZ] = "";
 
 /* read word from freedom.conf */
 #define read_conf(t) \
-if (!strncmp(line, #t, sizeof(#t) - 1)) \
-  sscanf(line, "%s %s", junk, t)
+if (!strncmp(line, #t, sizeof(#t) - 1) && isspace(line[sizeof(#t) - 1])) \
+  sscanf(line, "%s %s", junk, t);
 
 /* read line from freedom.conf, with whitespace */
 #define read_conf_w(t) \
-if (!strncmp(line, #t, sizeof(#t) - 1)) {\
+if (!strncmp(line, #t, sizeof(#t) - 1) && isspace(line[sizeof(#t) - 1])) {\
   foo = line;\
   while (*foo > ' ') foo++; /* scan to first space */ \
   while (*foo > '\0' && *foo <= ' ') foo++; /* scan to first non whitespace */ \
@@ -84,7 +89,7 @@ if (!strncmp(line, #t, sizeof(#t) - 1)) {\
 
 /* read number from freedom.conf */
 #define read_conf_i(t) \
-if (!strncmp(line, #t, sizeof(#t) - 1)) \
+if (!strncmp(line, #t, sizeof(#t) - 1) && isspace(line[sizeof(#t) - 1])) \
   sscanf(line, "%s %d", junk, &t);
 
 int
@@ -104,7 +109,7 @@ init_remailer(void)
   }
 
   if (!getcwd(cur_dir, sizeof(cur_dir))) { /*could not get current dir */
-    fprintf(stderr, "Could not get current directory!\n");
+    fputs("Could not get current directory!\n", stderr);
     strcpy(cur_dir, remailer_dir); /* So set cur_dir to remailer_dir */
   }
   if (cur_dir[strlen(cur_dir) - 1] != '/')
@@ -129,13 +134,13 @@ init_remailer(void)
   srand((time(NULL) ^ (getpid() | (getpid() << 16))) & RAND_MAX);
 
   if (!(infile = fopen("freedom.conf", "r"))) {
-    fprintf (stderr, "Error: Could not open freedom.conf!\n");
+    fputs("Error: Could not open freedom.conf!\n", stderr);
     return -1;
   }
   else {
     while (fgets(line, sizeof(line), infile)) {
       if (line[0] != '#' && strlen(line) > 1) {
-	if (!strncmp(line, "REMAILER_DIR", 12)) {
+	if (!strncmp(line, "REMAILER_DIR", sizeof("REMAILER_DIR") - 1)) {
 	  sscanf(line, "%s %s", junk, remailer_dir);
 	  if (chdir(remailer_dir))
 	    fprintf(stderr, "Error changing to REMAILER_DIR %s\n",
@@ -145,6 +150,7 @@ init_remailer(void)
 	read_conf_i(USE_STATS);
 	read_conf_i(USE_MIX);
 	read_conf_i(ALLOW_PGP);
+	read_conf_i(ALLOW_GPG);
 	read_conf_i(ALLOW_POST);
 	read_conf_i(ALLOW_WWW);
 	read_conf_i(ALLOW_LTIME);
@@ -152,18 +158,21 @@ init_remailer(void)
 	read_conf(MIX_DIR);
 	read_conf(QUEUE_DIR);
 	read_conf(PGP_DIR);
+	read_conf(GPG_DIR);
 	read_conf(MAIL_SPOOL);
 	read_conf_w(SENDMAIL);
 	read_conf_w(INEWS);
+	read_conf(MIXMASTER);
 	read_conf(PGP);
+	read_conf(GPG);
 	read_conf(WGET);
 	read_conf(HELP_FILE);
 	read_conf(STATS_FILE);
 	read_conf(KEY_FILE);
 	read_conf(PASS_FILE);
+	read_conf(TYPE2_LIST);
 	read_conf(SOURCE_BLOCK);
 	read_conf(DEST_BLOCK);
-	read_conf(SUBJ_BLOCK);
 	read_conf(GROUP_BLOCK);
 	read_conf(URL_BLOCK);
 	read_conf(HDRFILTER);
@@ -176,13 +185,27 @@ init_remailer(void)
 	read_conf(COMPLAINTS);
       }
     }
-    if (ALLOW_LTIME && (QUEUE_DIR[0] == '\0'))
+    if (ALLOW_LTIME && QUEUE_DIR[0] == '\0')
       strcpy(QUEUE_DIR, REMAILER_DIR "/queue");
+    if (USE_MIX) {
+      if (MIXMASTER[0] == '\0') {
+	strcpy(MIXMASTER, MIX_DIR);
+	strncat(MIXMASTER, "/mixmaster", sizeof(MIXMASTER) -
+		strlen(MIXMASTER) - 1);
+      }
+      if (TYPE2_LIST[0] == '\0') {
+	strcpy(TYPE2_LIST, MIX_DIR);
+	strncat(TYPE2_LIST, "/type2.list", sizeof(TYPE2_LIST) -
+		strlen(TYPE2_LIST) - 1);
+      }
+    }
     if (ALLOW_PGP) {
       if (PGP_DIR[0] == '\0')
 	strcpy(PGP_DIR, REMAILER_DIR "/.pgp");
-      if (PASS_FILE[0] == '\0')
-	strcpy(PASS_FILE, REMAILER_DIR "/.pgp/passphrase");
+    }
+    if (ALLOW_GPG) {
+      if (GPG_DIR[0] == '\0')
+	strcpy(GPG_DIR, REMAILER_DIR "/.gnupg");
     }
     if (ANON_NAME[0] == '\0')
       strcpy(ANON_NAME, REMAILER_NAME);
@@ -205,14 +228,14 @@ main(int argc, char *argv[])
   char *send_to = alloca(BUFSIZ);
   char *message_from = alloca(BUFSIZ);
   int recursing, trimmings = 0;
-  int tempfd;
+  int tempfd, poolfd;
   enum message_type anon;
-  struct stat tempfilestat;
+  struct stat filestat;
 
   init_remailer();
 
   if (USE_SYSLOG)
-    openlog(basename(argv[0]), LOG_PID, SYSLOG_FACILITY);
+    openlog(REMAILER_BIN, LOG_PID, SYSLOG_FACILITY);
 
   if (argc > 1) {
     if (!strcmp(argv[1], "-q")) {
@@ -226,14 +249,14 @@ main(int argc, char *argv[])
 	     "(C) John B. Fleming 1995-1997, Johannes Kroeger 1997-1998\n\n"
 	     "Usage: %s\t\t# read mail on stdin and process\n"
 	     "       %s -q\t# run queue of latent messages\n",
-	     VERSION, basename(argv[0]), basename(argv[0]));
+	     VERSION, REMAILER_BIN, REMAILER_BIN);
       exit(1);
     }
   }
 
   do /* make sure we don't overwrite other messages */
     sprintf(tempfilename, "mail%08x", rand());
-  while ((tempfd = open(tempfilename, O_WRONLY | O_CREAT | O_EXCL, 0660)) < 0);
+  while ((tempfd = open(tempfilename, O_WRONLY | O_CREAT | O_EXCL, 0600)) < 0);
   sprintf(bFilename, "%sB", tempfilename);
   sprintf(dFilename, "%sD", tempfilename);
   sprintf(eFilename, "%sE", tempfilename);
@@ -244,7 +267,7 @@ main(int argc, char *argv[])
   /* copy message coming in on stdin to a temporary file */
   if ((outfile = fdopen(tempfd, "w"))) {
     if (USE_SYSLOG)
-      syslog(LOG_DEBUG, "Writing new message to %s", basename(tempfilename));
+      syslog(LOG_DEBUG, "Writing new message to %s", tempfilename);
     while (fgets(line, sizeof(line), stdin))
       fputs(line, outfile);
     fclose(outfile);
@@ -256,11 +279,10 @@ main(int argc, char *argv[])
   }
 
   if (SIZE_LIMIT) {
-    stat(tempfilename, &tempfilestat);
-    if (tempfilestat.st_size > SIZE_LIMIT) {
+    stat(tempfilename, &filestat);
+    if (filestat.st_size > SIZE_LIMIT) {
       if (USE_SYSLOG) {
-	syslog(LOG_WARNING, "Message too large: %ld bytes",
-	       tempfilestat.st_size);
+	syslog(LOG_WARNING, "Message too large: %ld bytes", filestat.st_size);
 	syslog(LOG_DEBUG, "Deleting %s", tempfilename);
       }
       unlink(tempfilename);
@@ -270,7 +292,7 @@ main(int argc, char *argv[])
 
   recursing = 1;
   while (recursing) {
-    anon = scan_message(tempfilename, send_to, message_from, recursing);
+    anon = scan_message(tempfilename, send_to, recursing);
     recursing = 0;
 
     switch (anon) {
@@ -301,16 +323,32 @@ main(int argc, char *argv[])
 	fclose(outfile);
       }
       fclose(infile);
-      pgpdecrypt(eFilename, dFilename);
+      outfile = fopen(dFilename, "w");
+      if ((infile = fopen(eFilename, "r"))) {
+	run_cmd(PGP, "-d", NULL, infile, outfile);
+	fclose(infile);
+      }
+      rewind(outfile);
+      /* PGP 2.6.3in can't decrypt GNUPG messages but returns 0, so we
+	 check if dFilename is empty and try to decrypt eFilename with
+	 GNUPG if ALLOW_GPG != 0 */
+      stat(dFilename, &filestat);
+      if (filestat.st_size == 0 && ALLOW_GPG) {
+	if ((infile = fopen(eFilename, "r"))) {
+	  run_cmd(GPG, "-d", NULL, infile, outfile);
+	  fclose(infile);
+	}
+      }
+      fclose(outfile);
       rename(tempfilename, eFilename);
       outfile = fopen(tempfilename, "w");
       if ((infile = fopen(dFilename, "r"))) {
-	fputs("PGP-Recursive: Yes\n\n", outfile);
+	fputs("Recursive: PGP\n\n", outfile);
 	while (fgets(line, sizeof(line), infile))
 	  fputs(line, outfile);
 	fclose(infile);
       }
-      /* Text after the first PGP packet is appended to decrypted message */
+      /* append lines after the first PGP packet to decrypted message */
       if ((infile = fopen(eFilename, "r"))) {
 	while (fgets(line, sizeof(line), infile)) {
 	  if (!strcmp(line, "-----END PGP MESSAGE-----\n")) {
@@ -326,102 +364,211 @@ main(int argc, char *argv[])
       recursing = 1;
     break;
 
-    case ANON_MESSAGE:
+    case REMIX_MESSAGE:
+      if (USE_SYSLOG) {
+	syslog(LOG_INFO, "Type 2 destination detected");
+      }
+      outfile = fopen(bFilename, "w");
+      fputs(send_to, outfile);
+      fputs("\n\n", outfile);
+      fflush(outfile);
+      if ((infile = fopen(tempfilename, "r"))) {
+	remove_headers(infile, outfile, anon);
+	fclose(infile);
+      }
+      fclose(outfile);
+      unlink(tempfilename);
       if (USE_SYSLOG)
-	syslog(LOG_INFO, "Anonymous message detected");
-      remove_headers(tempfilename, bFilename, send_to);
-      if (scan_subject(bFilename) == -1) {
+	syslog(LOG_INFO, "Reordering message");
+      do /* make sure we don't overwrite other messages */
+	sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
+      while ((poolfd = open(poolfilename, O_WRONLY | O_CREAT | O_EXCL, 0600))
+	     < 0);
+      outfile = fdopen(poolfd, "a");
+      fputs("-----Intermediate Hop-----\n", outfile);
+      fputs(send_to, outfile);
+      fputs("\nEND\n", outfile);
+      fflush(outfile);
+      if ((infile = fopen(bFilename, "r"))) {
 	if (USE_SYSLOG)
-	  syslog(LOG_NOTICE, "Discarding blocked message");
+	  syslog(LOG_DEBUG, "Remixing message to %s", send_to);
+	run_cmd(MIXMASTER, "-l", send_to, infile, outfile);
+	fclose(infile);
+      }
+      fclose(outfile);
+      unlink(bFilename);
+    break;
+
+    case REPGP_MESSAGE:
+      if (USE_SYSLOG) {
+	syslog(LOG_INFO, "Type 1 destination detected");
+      }
+      outfile = fopen(bFilename, "w");
+      if ((infile = fopen(tempfilename, "r"))) {
+	remove_headers(infile, outfile, anon);
+	fclose(infile);
+      }
+      fclose(outfile);
+      unlink(tempfilename);
+      if (USE_MIX) {
+	if (USE_SYSLOG)
+	  syslog(LOG_INFO, "Reordering message");
+	do /* make sure we don't overwrite other messages */
+	  sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
+	while ((poolfd = open(poolfilename, O_WRONLY | O_CREAT | O_EXCL, 0600))
+	       < 0);
+	outfile = fdopen(poolfd, "a");
+	fputs("-----Final Hop-----\n", outfile);
+	fputs(send_to, outfile);
+	fputs("\nEND\n\n::\nEncrypted: PGP\n\n", outfile);
+	fflush(outfile);
+	if ((infile = fopen(bFilename, "r"))) {
+	  if (USE_SYSLOG)
+	    syslog(LOG_DEBUG, "Encrypting message to %s", send_to);
+	  run_cmd(PGP, "-e", send_to, infile, outfile);
+	  fclose(infile);
+	}
+	fclose(outfile);
 	unlink(bFilename);
       }
       else {
 	if (USE_SYSLOG)
 	  syslog(LOG_INFO, "Remailing message");
-	if (USE_MIX == 1) {
-	  do /* make sure we don't overwrite other messages */
-	    sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
-	  while (link(bFilename, poolfilename) < 0);
-	}
-	else {
-	  if (!USE_MIX)
-	    sprintf(line, "%s", SENDMAIL);
-	  else if (USE_MIX == 2)
-	    sprintf(line, "%s/%s", MIX_DIR, MIX_MAIL);
-	  p = popen(line, "w");
-	  if (!USE_MIX) {
-	    fprintf(p, "From: %s <%s>\n", ANON_NAME, ANON_ADDR);
-	    fprintf(p, DISCLAIMER);
-	  }
-	  if ((infile = fopen(bFilename, "r"))) {
-	    while (fgets(line, sizeof(line), infile))
-	      fputs(line, p);
-	    fclose(infile);
-	  }
-	  pclose(p);
+	sprintf(line, "%s", SENDMAIL);
+	p = popen(line, "w");
+	fprintf(p, "From: %s <%s>\n", ANON_NAME, ANON_ADDR);
+	fprintf(p, DISCLAIMER);
+	fputs("To: ", p);
+	fputs(send_to, p);
+	fputs("\n\n::\nEncrypted: PGP\n\n", p);
+	fflush(p);
+	if ((infile = fopen(bFilename, "r"))) {
+	  if (USE_SYSLOG)
+	    syslog(LOG_DEBUG, "Encrypting message to %s", send_to);
+	  run_cmd(PGP, "-e", send_to, infile, p);
+	  fclose(infile);
 	}
+	pclose(p);
 	unlink(bFilename);
       }
     break;
 
+    case ANON_MESSAGE:
+      if (USE_SYSLOG)
+	syslog(LOG_INFO, "Anonymous message detected");
+      if (USE_MIX) {
+	if (USE_SYSLOG)
+	  syslog(LOG_INFO, "Reordering message");
+	do /* make sure we don't overwrite other messages */
+	  sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
+	while ((poolfd = open(poolfilename, O_WRONLY | O_CREAT | O_EXCL, 0600))
+	       < 0);
+	outfile = fdopen(poolfd, "w");
+	fputs("-----Final Hop-----\n", outfile);
+	fputs(send_to, outfile);
+	if (strlen(send_to) > 0 && send_to[strlen(send_to) - 1] != '\n') {
+	  if (USE_SYSLOG)
+	    syslog(LOG_WARNING, "Truncated destination buffer");
+	  fputs("\n", outfile);
+	}
+	fputs("END\n", outfile);
+	fflush(outfile);
+	if ((infile = fopen(tempfilename, "r"))) {
+	  remove_headers(infile, outfile, anon);
+	  fclose(infile);
+	}
+	fclose(outfile);
+      }
+      else {
+	if (USE_SYSLOG)
+	  syslog(LOG_INFO, "Remailing message");
+	sprintf(line, "%s", SENDMAIL);
+	p = popen(line, "w");
+	fprintf(p, "From: %s <%s>\n", ANON_NAME, ANON_ADDR);
+	fprintf(p, DISCLAIMER);
+	fputs(send_to, p);
+	if (strlen(send_to) > 0 && send_to[strlen(send_to) - 1] != '\n') {
+	  if (USE_SYSLOG)
+	    syslog(LOG_WARNING, "Truncated destination buffer");
+	  fputs("\n", p);
+	}
+	fflush(p);
+	if ((infile = fopen(tempfilename, "r"))) {
+	  remove_headers(infile, p, anon);
+	  fclose(infile);
+	}
+	pclose(p);
+      }
+      unlink(tempfilename);
+    break;
+
     case ANON_POST:
       if (USE_SYSLOG)
 	syslog(LOG_INFO, "Anonymous post detected");
-      remove_headers(tempfilename, bFilename, send_to);
-      if (scan_subject(bFilename) == -1) {
+      if (USE_MIX) {
 	if (USE_SYSLOG)
-	  syslog(LOG_NOTICE, "Discarding blocked message");
-	unlink(bFilename);
+	  syslog(LOG_INFO, "Reordering message");
+	do /* make sure we don't overwrite other messages */
+	  sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
+	while ((poolfd = open(poolfilename, O_WRONLY | O_CREAT | O_EXCL, 0600))
+	       < 0);
+	outfile = fdopen(poolfd, "w");
+	fputs("-----Final Hop-----\n", outfile);
+	fputs(send_to, outfile);
+	if (strlen(send_to) > 0 && send_to[strlen(send_to) - 1] != '\n') {
+	  if (USE_SYSLOG)
+	    syslog(LOG_WARNING, "Truncated destination buffer");
+	  fputs("\n", outfile);
+	}
+	fputs("END\n", outfile);
+	fflush(outfile);
+	if ((infile = fopen(tempfilename, "r"))) {
+	  remove_headers(infile, outfile, anon);
+	  fclose(infile);
+	}
+	fclose(outfile);
       }
       else {
 	if (USE_SYSLOG)
 	  syslog(LOG_INFO, "Posting message");
-	if (USE_MIX == 1) {
-	  do /* make sure we don't overwrite other messages */
-	    sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
-	  while (link(bFilename, poolfilename) < 0);
+	if (ALLOW_POST == 1)
+	  sprintf(line, "%s", INEWS);
+	else if (ALLOW_POST == 2)
+	  sprintf(line, "%s", SENDMAIL);
+	p = popen(line, "w");
+	if (ALLOW_POST == 1)
+	  fprintf(p, "Organization: %s\n", ORGANIZATION);
+	else if (ALLOW_POST == 2)
+	  fprintf(p, "To: %s\n", MAIL2NEWS);
+	fprintf(p, "From: %s <%s>\n", ANON_NAME, ANON_ADDR);
+	fprintf(p, DISCLAIMER);
+	fputs(send_to, p);
+	if (strlen(send_to) > 0 && send_to[strlen(send_to) - 1] != '\n') {
+	  if (USE_SYSLOG)
+	    syslog(LOG_WARNING, "Truncated destination buffer");
+	  fputs("\n", p);
 	}
-	else {
-	  if (!USE_MIX) {
-	    if (ALLOW_POST == 1)
-	      sprintf(line, "%s", INEWS);
-	    else if (ALLOW_POST == 2)
-	      sprintf(line, "%s", SENDMAIL);
-	  }
-	  else if (USE_MIX == 2)
-	    sprintf(line, "%s/%s", MIX_DIR, MIX_NEWS);
-	  p = popen(line, "w");
-	  if (!USE_MIX) {
-	    if (ALLOW_POST == 1)
-	      fprintf(p, "Organization: %s\n", ORGANIZATION);
-	    else if (ALLOW_POST == 2)
-	      fprintf(p, "To: %s\n", MAIL2NEWS);
-	    fprintf(p, "From: %s <%s>\n", ANON_NAME, ANON_ADDR);
-	    fprintf(p, DISCLAIMER);
-	    if (!scan_subject(bFilename))
-	      fprintf(p, "Subject: none\n");
-	  }
-	  if ((infile = fopen(bFilename, "r"))) {
-	    while (fgets(line, sizeof(line), infile))
-	      fputs(line, p);
-	    fclose(infile);
-	  }
-	  pclose(p);
+	fflush(p);
+	if ((infile = fopen(tempfilename, "r"))) {
+	  remove_headers(infile, p, anon);
+	  fclose(infile);
 	}
-	unlink(bFilename);
+	pclose(p);
       }
+      unlink(tempfilename);
     break;
 
     case STATS_REQ:
       if (USE_SYSLOG)
 	syslog(LOG_INFO, "Statistics were requested");
-      if (blocked(message_from, DEST_BLOCK)) {
+      get_from(tempfilename, message_from);
+      if (rxfind(message_from, DEST_BLOCK)) {
 	if (USE_SYSLOG)
 	  syslog(LOG_NOTICE, "Ignoring request");
       }
       else {
 	if (USE_SYSLOG)
-	  syslog(LOG_INFO, "Mailing statistics");
+	  syslog(LOG_INFO, "Mailing statistics to %s", message_from);
 	mailstats(message_from);
       }
       unlink(tempfilename);
@@ -430,13 +577,14 @@ main(int argc, char *argv[])
     case CONF_REQ:
       if (USE_SYSLOG)
 	syslog(LOG_INFO, "Abilities were requested");
-      if (blocked(message_from, DEST_BLOCK)) {
+      get_from(tempfilename, message_from);
+      if (rxfind(message_from, DEST_BLOCK)) {
 	if (USE_SYSLOG)
 	  syslog(LOG_NOTICE, "Ignoring request");
       }
       else {
 	if (USE_SYSLOG)
-	  syslog(LOG_INFO, "Mailing abilities");
+	  syslog(LOG_INFO, "Mailing abilities to %s", message_from);
 	mailconf(message_from);
       }
       unlink(tempfilename);
@@ -445,13 +593,15 @@ main(int argc, char *argv[])
     case HELP_REQ:
       if (USE_SYSLOG)
 	syslog(LOG_INFO, "Help was requested");
+      get_from(tempfilename, message_from);
       mailfile(HELP_FILE, message_from);
       unlink(tempfilename);
     break;
 
     case KEY_REQ:
       if (USE_SYSLOG)
-	syslog(LOG_INFO, "PGP key was requested");
+	syslog(LOG_INFO, "Keys were requested");
+      get_from(tempfilename, message_from);
       mailfile(KEY_FILE, message_from);
       unlink(tempfilename);
     break;
@@ -459,7 +609,7 @@ main(int argc, char *argv[])
     case CUTMARKS:
       if (USE_SYSLOG)
 	syslog(LOG_DEBUG, "Cutmarks: %s detected", send_to);
-      cut(tempfilename, cFilename, send_to, "cutmarks:", 1);
+      cut(tempfilename, cFilename, send_to, "cutmarks:");
       trimmings = 1;
     break;
 
@@ -480,8 +630,14 @@ main(int argc, char *argv[])
     case ENCRYPT_KEY:
       if (USE_SYSLOG)
 	syslog(LOG_DEBUG, "Encrypt-Key: %s detected", send_to);
-      cut(tempfilename, kFilename, "**", "encrypt-key:", 0);
-      encrypt_key(kFilename, tempfilename, send_to);
+      cut(tempfilename, kFilename, "**", "encrypt-key:");
+      outfile = fopen(tempfilename, "a");
+      if ((infile = fopen(kFilename, "r"))) {
+	run_cmd(PGP, "-c", send_to, infile, outfile);
+	fclose(infile);
+      }
+      fclose(outfile);
+      unlink(kFilename);
       recursing = 1;
     break;
 
@@ -506,13 +662,13 @@ main(int argc, char *argv[])
 	if ((infile = fopen(tempfilename, "r"))) {
 	  fgets(line, sizeof(line), infile);
 	  rewind(infile);
-	  if (!strcmp(line, "Cut-Recursive: Yes\n")) {
+	  if (!strcmp(line, "Recursive: Cut\n")) {
 	    if (USE_SYSLOG) {
 	      syslog(LOG_DEBUG, "It's cutmarks leftovers");
 	      syslog(LOG_INFO, "Discarding undeliverable message");
 	    }
 	  }
-	  else if (!strcmp(line, "PGP-Recursive: Yes\n")) {
+	  else if (!strcmp(line, "Recursive: PGP\n")) {
 	    if (USE_SYSLOG) {
 	      syslog(LOG_DEBUG, "Couldn't decrypt message");
 	      syslog(LOG_INFO, "Discarding undeliverable message");
