@@ -44,17 +44,17 @@ updatestats(enum stats_flag flag)
   curtime = localtime(&now);
 
   /* don't manipulate the stats file before locking stats.lock exclusively */
-  lockfd = open("stats.lock", O_WRONLY | O_CREAT | O_TRUNC, 0660);
+  lockfd = open("stats.lock", O_WRONLY | O_CREAT | O_TRUNC, 0600);
   flock(lockfd, LOCK_EX);
   write(lockfd, &pid, sizeof(pid_t));
 
   if (!(file = fopen(STATS_FILE, "r"))) {
     /* STATS_FILE does not exist; make a new empty stats file */
     file = fopen(STATS_FILE, "w+");
-    fprintf(file, "0\n");
-    fprintf(file, "0 0\n");
+    fputs("0\n", file);
+    fputs("0 0\n", file);
     for (x = 0; x < 24; x++) /* tmp for month. First day, no messages */
-      fprintf(file, "tmp 0 0 0\n");
+      fputs("tmp 0 0 0\n", file);
     rewind(file);
   }
   fscanf(file, "%d", &hour);
@@ -120,7 +120,7 @@ mailstats(const char *to)
 
 
   /* don't manipulate the stats file before locking stats.lock exclusively */
-  lockfd = open("stats.lock", O_WRONLY | O_CREAT | O_TRUNC, 0660);
+  lockfd = open("stats.lock", O_WRONLY | O_CREAT | O_TRUNC, 0600);
   flock(lockfd, LOCK_EX);
   write(lockfd, &pid, sizeof(pid_t));
 
@@ -156,10 +156,10 @@ mailstats(const char *to)
     sprintf(sendmail, "%s", SENDMAIL);
     file = popen(sendmail, "w");
     fprintf(file, "From: %s <%s>\n", REMAILER_NAME, REMAILER_ADDR);
-    fprintf(file, "Subject: Cypherpunk Remailer Statistics\n");
-    fprintf(file, "To: %s\n\n", to);
-    fprintf(file, "Statistics for last 24 hours from anonymous remailer\n");
-    fprintf(file, "%s\n\n", REMAILER_NAME);
+    fprintf(file, "Subject: Cypherpunk Remailer Statistics\n"
+		  "To: %s\n\n", to);
+    fprintf(file, "Statistics for last 24 hours from anonymous remailer\n"
+		  "%s\n\n", REMAILER_NAME);
     fprintf(file, "Number of messages per hour from "
 		  "%s %2d %02d:00 to %s %2d %02d:59\n\n",
 		  month[23], date[23], currenthour, month[0], date[0],
@@ -172,19 +172,18 @@ mailstats(const char *to)
       fprintf(file, "%02d:00 (%3d) ", x, Hm[x]);
       if (Hm[x] > 0) {
 	for (y = 0; y < Hp[x] * mscale; y++)
-	  fprintf (file, "*");
+	  fputs("*", file);
 	for (y = 0; y < (Hm[x] - Hp[x]) * mscale; y++)
-	  fprintf (file, "+");
+	  fputs("+", file);
 	Tm += Hm[x];
 	Tp += Hp[x];
       }
-      fprintf(file, "\n");
+      fputs("\n", file);
     }
-    fprintf(file, "\n");
-    fprintf(file, "Total messages received in last 24 hours: \t%5d\n", Tm);
+    fprintf(file, "\nTotal messages received in last 24 hours:\t%5d\n", Tm);
     if (ALLOW_PGP)
-      fprintf(file, "Number of PGP encrypted messages: \t\t%5d\n", Tp);
-    fprintf(file, "Number of plaintext messages: \t\t\t%5d\n", Tm - Tp);
+      fprintf(file, "Number of PGP encrypted messages:\t\t%5d\n", Tp);
+    fprintf(file, "Number of plaintext messages:\t\t\t%5d\n", Tm - Tp);
     pclose(file);
   }
   flock(lockfd, LOCK_UN);
@@ -206,69 +205,99 @@ mailconf(const char *to)
   fprintf(p, "Remailer-Type: Freedom %s\n\n", VERSION);
 
   if (SIZE_LIMIT)
-    fprintf(p, "Maximum message size: %d bytes\n", SIZE_LIMIT);
+    fprintf(p, "Maximum message size:\t\t\t\t%d bytes\n", SIZE_LIMIT);
   else
-    fprintf(p, "Maximum message size: unlimited\n");
-
-  if (USE_MIX)
-    fprintf(p, "Using Mixmaster reordering pool: supported\n");
-  else
-    fprintf(p, "Using Mixmaster reordering pool: not supported\n");
-
-  if (ALLOW_LTIME)
-    fprintf(p, "Queuing messages with Latent-Time: supported\n");
-  else
-    fprintf(p, "Queuing messages with Latent-Time: not supported\n");
+    fputs("Maximum message size:\t\t\t\tunlimited\n", p);
 
   if (USE_STATS)
-    fprintf(p, "Message statistics autoresponder: supported\n");
+    fputs("Message statistics autoresponder:\t\tenabled\n", p);
   else
-    fprintf(p, "Message statistics autoresponder: not supported\n");
+    fputs("Message statistics autoresponder:\t\tdisabled\n", p);
+
+  if (ALLOW_LTIME)
+    fputs("Queuing messages with Latent-Time:\t\tenabled\n\n", p);
+  else
+    fputs("Queuing messages with Latent-Time:\t\tdisabled\n\n", p);
+
+#ifdef USE_RX
+    fputs("Blocked header lines (regular expressions):\n", p);
+#else /* !USE_RX */
+    fputs("Blocked header lines:\n", p);
+#endif /* !USE_RX */
+    print_list(p, HDRFILTER);
+    fputs("\n", p);
+
+  if (ALLOW_PGP == 2)
+    fputs("Unencrypted incoming messages:\t\t\tnot supported\n", p);
+  else
+    fputs("Unencrypted incoming messages:\t\t\tsupported\n", p);
 
   if (ALLOW_PGP) {
-    fprintf(p, "PGP encryption of incoming messages: supported\n");
-    fprintf(p, "Message encryption with Encrypt-Key: supported\n");
+    fputs("PGP-encrypted incoming messages:\t\tsupported\n", p);
+    fputs("Conventional encryption with Encrypt-Key:\tsupported\n", p);
+    fputs("Subject encryption with Encrypt-Subject:\tsupported\n", p);
+    fputs("Encryption to next hop with Encrypt-To:\t\tsupported\n\n", p);
+    fputs("Cypherpunk remailer keys available for Encrypt-To:", p);
+    fflush(p);
+    run_cmd(PGP, "-kv", NULL, NULL, p);
+    fputs("\n", p);
   }
   else {
-    fprintf(p, "PGP encryption of incoming messages: not supported\n");
-    fprintf(p, "Message encryption with Encrypt-Key: not supported\n");
+    fputs("PGP-encrypted incoming messages:\t\tnot supported\n", p);
+    fputs("Conventional encryption with Encrypt-Key:\tnot supported\n", p);
+    fputs("Subject encryption with Encrypt-Subject:\tsupported\n", p);
+    fputs("Encryption to next hop with Encrypt-To:\t\tnot supported\n\n", p);
   }
 
-  fprintf(p, "Subject encryption with Encrypt-Subject: supported\n");
-
-  if (ALLOW_WWW) {
-    fprintf(p, "Retrieving WWW files with Get-URL: supported\n");
-#ifdef USE_RX
-    fprintf(p, "Blocked URLs (regular expressions):\n");
-#else /* !USE_RX */
-    fprintf(p, "Blocked URLs:\n");
-#endif /* !USE_RX */
-    print_blocked(p, URL_BLOCK);
-  }
+  if (ALLOW_GPG)
+    fputs("GNUPG-encrypted incoming messages:\t\tsupported\n\n", p);
   else
-    fprintf(p, "Retrieving WWW files with Get-URL: not supported\n");
+    fputs("GNUPG-encrypted incoming messages:\t\tnot supported\n\n", p);
+
+  if (USE_MIX == 2)
+    fputs("Transparent remixing to next hop:\t\tenabled\n", p);
+  else
+    fputs("Transparent remixing to next hop:\t\tdisabled\n", p);
+
+  if (USE_MIX) {
+    fputs("Reordering through Mixmaster pool:\t\tenabled\n", p);
+    fputs("Remixing to next hop with Remix-To:\t\tenabled\n\n", p);
+    fputs("Mixmaster remailer keys available for Remix-To:\n", p);
+    print_list(p, TYPE2_LIST);
+    fputs("\n", p);
+  }
+  else {
+    fputs("Reordering through Mixmaster pool:\t\tdisabled\n", p);
+    fputs("Remixing to next hop with Remix-To:\t\tdisabled\n\n", p);
+  }
 
   if (ALLOW_POST) {
     if (ALLOW_POST == 1)
-      fprintf(p, "News posting with Anon-Post-To: via local news server\n");
+      fputs("News posting with Anon-Post-To:\t\tvia local news server\n\n", p);
     else if (ALLOW_POST == 2)
-      fprintf(p, "News posting with Anon-Post-To: via %s\n", MAIL2NEWS);
+      fprintf(p, "News posting with Anon-Post-To:\t\tvia %s\n\n", MAIL2NEWS);
 #ifdef USE_RX
-    fprintf(p, "Blocked newsgroups (regular expressions):\n");
+    fputs("Blocked newsgroups (regular expressions):\n", p);
 #else /* !USE_RX */
-    fprintf(p, "Blocked newsgroups:\n");
+    fputs("Blocked newsgroups:\n", p);
 #endif /* !USE_RX */
-    print_blocked(p, GROUP_BLOCK);
+    print_list(p, GROUP_BLOCK);
+    fputs("\n", p);
   }
   else
-    fprintf(p, "News posting with Anon-Post-To: not supported\n");
+    fputs("News posting with Anon-Post-To:\t\t\tdisabled\n\n", p);
 
+  if (ALLOW_WWW) {
+    fputs("Retrieving WWW files with Get-URL:\t\tenabled\n\n", p);
 #ifdef USE_RX
-    fprintf(p, "Blocked subjects (regular expressions):\n");
+    fputs("Blocked URLs (regular expressions):\n", p);
 #else /* !USE_RX */
-    fprintf(p, "Blocked subjects:\n");
+    fputs("Blocked URLs:\n", p);
 #endif /* !USE_RX */
-    print_blocked(p, SUBJ_BLOCK);
+    print_list(p, URL_BLOCK);
+  }
+  else
+    fputs("Retrieving WWW files with Get-URL:\t\tdisabled\n", p);
 
   pclose(p);
 }
