@@ -27,27 +27,25 @@
 #include "freedom.h"
 
 #ifdef USE_RX
-#include <rxposix.h>	/* for use with GNU Rx regexp library */
-/* #include <rx.h> */	/* for use with libc regexp functions */
+#include "pcre.h"
 
 int
 rxmatch(const char *string, const char *pattern)
 {
-  int err, match;
-  char errmsg[80];
-  regex_t compiled;
+  int errptr, match;
+  const char *error;
+  pcre *compiled;
 
-  if ((err = regcomp(&compiled, pattern, REG_EXTENDED | REG_ICASE))) {
-    regerror(err, &compiled, errmsg, sizeof(errmsg));
-    if (USE_SYSLOG)
-      syslog(LOG_WARNING, "Couldn't compile regex %s: %s", pattern, errmsg);
-    regfree(&compiled);
-    return 0;
+  if ((compiled = pcre_compile(pattern, PCRE_CASELESS, &error, &errptr))) {
+    match = pcre_exec(compiled, NULL, string, strlen(string),
+		      PCRE_CASELESS, NULL, 0);
+    free(compiled);
+    return (match >= 0);
   }
   else {
-    match = regexec(&compiled, string, 0, NULL, 0);
-    regfree(&compiled);
-    return !match;
+    if (USE_SYSLOG)
+      syslog(LOG_WARNING, "Error in regex at offset %d: %s", errptr, error);
+    return 0;
   }
 }
 #else /* !USE_RX */
@@ -59,17 +57,28 @@ rxmatch(const char *string, const char *pattern)
 #endif /* !USE_RX */
 
 void
-chop(char *string)
+flushleft(char *string, const unsigned pos)
 {
   unsigned i, j;
 
-  for (i = 0, j = 0; i < strlen(string) && string[i] != '\n'; i++) {
-    if((j > 0) || !isspace(string[i])) {
+  for (i = pos, j = 0; i <= strlen(string); i++) {
+    if(j > 0 || !isspace(string[i])) {
       string[j] = string[i];
       j++;
     }
   }
-  string[j] = '\0';
+}
+
+void
+chop(char *string)
+{
+  size_t l = strlen(string);
+
+  if (l > 0 && string[l - 1] == '\n') {
+    string[l - 1] = '\0';
+    if (l > 1 && string[l - 2] == '\r')
+      string[l - 2] = '\0';
+  }
 }
 
 int
@@ -115,15 +124,15 @@ strieq(const char *s1, const char *s2)
 }
 
 int
-blocked(const char *address, const char *blockfilename)
+rxfind(const char *address, const char *listname)
 {
   FILE *infile;
   char line[BUFSIZ];
-  int block = 0;
+  int match = 0;
 
-  if (!(infile = fopen(blockfilename, "r"))) {
+  if (!(infile = fopen(listname, "r"))) {
     if (USE_SYSLOG)
-      syslog(LOG_WARNING, "Can't open %s: %m", blockfilename);
+      syslog(LOG_WARNING, "Can't open %s: %m", listname);
     return 0;
   }
 
@@ -133,25 +142,48 @@ blocked(const char *address, const char *blockfilename)
       continue; /* skip blank or comment lines */
     else if (rxmatch(address, line)) {
       if (USE_SYSLOG)
-	syslog(LOG_NOTICE, "%s matches %s in %s",
-	       address, line, basename(blockfilename));
-      block = 1;
+	syslog(LOG_NOTICE, "%s matches %s in %s", address, line, listname);
+      match = 1;
       break;
     }
   }
   fclose(infile);
-  return block;
+  return match;
 }
 
 int
-print_blocked(FILE *outfile, const char *blockfilename)
+findline(const char *address, const char *listname)
+{
+  FILE *infile;
+  char line[BUFSIZ];
+  int linenum = 0;
+
+    if (!(infile = fopen(listname, "r"))) {
+    if (USE_SYSLOG)
+      syslog(LOG_WARNING, "Can't open %s: %m", listname);
+    return 0;
+  }
+
+  while (fgets(line, sizeof(line), infile)) {
+    linenum++;
+    if (strifind(line, address)) {
+      fclose(infile);
+      return linenum;
+    }
+  }
+  fclose(infile);
+  return 0;
+}
+
+int
+print_list(FILE *outfile, const char *listname)
 {
   FILE *infile;
   char line[BUFSIZ];
 
-  if (!(infile = fopen(blockfilename, "r"))) {
+  if (!(infile = fopen(listname, "r"))) {
     if (USE_SYSLOG)
-      syslog(LOG_WARNING, "Can't open %s: %m", blockfilename);
+      syslog(LOG_WARNING, "Can't open %s: %m", listname);
     return 1;
   }
 
@@ -166,7 +198,7 @@ print_blocked(FILE *outfile, const char *blockfilename)
 
 void
 cut(const char *origfilename, const char *cutfilename, const char *cutmarks,
-    const char *header, const int fakeheader)
+    const char *header)
 {
   char tempfilename[BUFSIZ], line[BUFSIZ], tmpline[BUFSIZ];
   FILE *infile, *outfile;
@@ -174,32 +206,32 @@ cut(const char *origfilename, const char *cutfilename, const char *cutmarks,
 
   sprintf(tempfilename, "%sT", origfilename);
   rename(origfilename, tempfilename);
-
+  outfile = fopen(origfilename, "w");
   if (!(infile = fopen(tempfilename, "r"))) {
-    outfile = fopen(origfilename, "w");
     fclose(outfile);
     return;
   }
-  outfile = fopen(origfilename, "w");
 
   while (fgets(line, sizeof(line), infile)) {
     strcpy(tmpline, line);
     chop(tmpline);
     if (!headers) {
-      if ((linenum > 1) || (strlen(line) > 1))
+      if (linenum > 1 || strlen(line) > 1)
 	linenum++;
-      if ((linenum == 1) && strstr(line, "::")) {
+      if (linenum == 1 && !strncmp(line, "::", 2)) {
 	headers = 1;
 	fputs(line, outfile);
       }
       else {
 	if (!cut && !strcmp(tmpline, cutmarks)) {
+	  if (!strcmp(header, "encrypt-key:"))
+	    fputs(line, outfile);
 	  fclose(outfile);
 	  if (USE_SYSLOG)
 	    syslog(LOG_DEBUG, "Cutting message");
 	  outfile = fopen(cutfilename, "w");
-	  if (fakeheader)
-	    fputs("Cut-Recursive: Yes\n\n", outfile);
+	  if (!strcmp(header, "cutmarks:"))
+	    fputs("Recursive: Cut\n\n", outfile);
 	  cut = 1;
 	}
 	else if (linenum > 0)
@@ -207,14 +239,10 @@ cut(const char *origfilename, const char *cutfilename, const char *cutmarks,
       }
     }
     else {
-      if (strlen(line) <= 1) {
+      if (!strileft(line, header))
+	fputs(line, outfile);
+      if (strlen(tmpline) == 0)
 	headers = 0;
-	fputs("\n", outfile);
-      }
-      else {
-	if (!strileft(line, header))
-	  fputs(line, outfile);
-      }
     }
   }
   fclose(infile);
@@ -228,13 +256,13 @@ mailfile(const char *filename, const char *to)
   FILE *infile, *outfile;
   char sendmail[BUFSIZ], line[BUFSIZ];
 
-  if (blocked(to, DEST_BLOCK)) {
+  if (rxfind(to, DEST_BLOCK)) {
     if (USE_SYSLOG)
       syslog(LOG_NOTICE, "Ignoring request");
   }
   else {
     if (USE_SYSLOG)
-      syslog(LOG_INFO, "Mailing %s", basename(filename));
+      syslog(LOG_INFO, "Mailing %s to %s", filename, to);
     sprintf(sendmail, "%s", SENDMAIL);
     if ((infile = fopen(filename, "r"))) {
       outfile = popen(sendmail, "w");
@@ -247,11 +275,3 @@ mailfile(const char *filename, const char *to)
     }
   }
 }
-
-/* get the basename of file <name> */
-const char *
-basename(const char *name)
-{
-  char *tmp = strrchr(name, '/');
-  return (tmp ? tmp + 1 : name);
-}
