@@ -49,8 +49,9 @@ runlatentqueue(void)
   if ((queuedir = opendir(QUEUE_DIR))) {
     while ((dir_entry = readdir(queuedir))) {
       strcpy(filename, dir_entry->d_name);
-      if (filename[0] == 'm') {
-	/* control file found - process it */
+      if (filename[0] == 'q') {
+	/* message found - process its control file */
+	filename[0] = 'm';
 	sprintf(queuefilename, "%s/%s", QUEUE_DIR, filename);
 	if ((infile = fopen(queuefilename, "r"))) {
 	  fgets(line, sizeof(line), infile);
@@ -64,8 +65,8 @@ runlatentqueue(void)
 	  stime = ctime;
 	}
 
-	if ((cday > sday) || ((cday == 0) && (sday == 6)) ||
-	    ((cday == sday) && (ctime >= stime))) {
+	if (cday > sday || (cday == 0 && sday == 6) ||
+	    (cday == sday && ctime >= stime)) {
 	  /* time to send the message */
 	  filename[0] = 'q';
 	  sprintf(queuefilename, "%s/%s", QUEUE_DIR, filename);
@@ -127,8 +128,8 @@ queuelatent(const char *origfilename, const char *latentTime)
     sprintf(qFilename, "%s/q%s", QUEUE_DIR, hexrand);
     sprintf(mFilename, "%s/m%s", QUEUE_DIR, hexrand);
   }
-  while ((qfd = open(qFilename, O_WRONLY | O_CREAT | O_EXCL, 0660)) < 0 ||
-	 (mfd = open(mFilename, O_WRONLY | O_CREAT | O_EXCL, 0660)) < 0);
+  while ((qfd = open(qFilename, O_WRONLY | O_CREAT | O_EXCL, 0600)) < 0 ||
+	 (mfd = open(mFilename, O_WRONLY | O_CREAT | O_EXCL, 0600)) < 0);
 
   if ((outfile = fdopen(mfd, "w"))) {
     fprintf(outfile, "%d\n%d\n", day, time2);
@@ -136,26 +137,26 @@ queuelatent(const char *origfilename, const char *latentTime)
   }
   if (!(infile = fopen(origfilename, "r"))) {
     if (USE_SYSLOG)
-      syslog(LOG_WARNING, "Can't open %s: %m", basename(origfilename));
+      syslog(LOG_WARNING, "Can't open %s: %m", origfilename);
     close(qfd);
     return;
   }
   outfile = fdopen(qfd, "w");
   if (USE_SYSLOG)
-    syslog(LOG_INFO, "Queuing message to %s", basename(qFilename));
+    syslog(LOG_INFO, "Queuing message to q%s", hexrand);
   while (fgets(line, sizeof(line), infile)) {
     if (!headers) {
-      if ((linenum > 1) || (strlen(line) > 1))
+      if (linenum > 1 || strlen(line) > 1)
 	linenum++;
-      if ((linenum == 1) && strstr(line, "::"))
+      if (linenum == 1 && !strncmp(line, "::", 2))
 	headers = 1;
       fputs(line, outfile);
     }
     else {
-      if (strlen(line) <= 1)
-	headers = 0;
       if (!strileft(line, "latent-time:"))
 	fputs(line, outfile);
+      if (strlen(line) <= 1)
+	headers = 0;
     }
   }
   fclose(infile);
@@ -170,7 +171,7 @@ parsetime(const char *latent, const int time_now)
   hour[2] = '\0'; minute[2] = '\0';
 
   if (isdigit(latent[0])) { /* no '+': parse as time given */
-    if (isdigit(latent[1]) && (latent[2] == ':') &&
+    if (isdigit(latent[1]) && latent[2] == ':' &&
 	isdigit(latent[3]) && isdigit(latent[4])) { /* two-digit hour */
       hour[0] = latent[0]; hour[1] = latent[1];
       minute[0] = latent[3]; minute[1] = latent[4];
@@ -182,7 +183,7 @@ parsetime(const char *latent, const int time_now)
       else /* hh:mm */
 	tim = ((atoi(hour) * 60) + atoi(minute)) % 1440;
     }
-    else if ((latent[1] == ':') &&
+    else if (latent[1] == ':' &&
 	     isdigit(latent[2]) && isdigit(latent[3])) { /* one-digit hour */
       hour[0] = latent[0]; hour[1] = '\0';
       minute[0] = latent[2]; minute[1] = latent[3];
@@ -198,7 +199,7 @@ parsetime(const char *latent, const int time_now)
       tim = time_now % 1440;
   }
   else if (latent[0] == '+') { /* '+': parse as delay */
-    if (isdigit(latent[1]) && isdigit(latent[2]) && (latent[3] == ':') &&
+    if (isdigit(latent[1]) && isdigit(latent[2]) && latent[3] == ':' &&
 	isdigit(latent[4]) && isdigit(latent[5])) { /* two-digit hour */
       hour[0] = latent[1]; hour[1] = latent[2];
       minute[0] = latent[4]; minute[1] = latent[5];
@@ -208,7 +209,7 @@ parsetime(const char *latent, const int time_now)
       else /* +hh:mm */
 	tim = (time_now + (atoi(hour) * 60) + atoi(minute)) % 1440;
     }
-    else if (isdigit(latent[1]) && (latent[2] == ':') &&
+    else if (isdigit(latent[1]) && latent[2] == ':' &&
 	     isdigit(latent[3]) && isdigit(latent[4])) { /* one-digit hour */
       hour[0] = latent[1]; hour[1] = '\0';
       minute[0] = latent[3]; minute[1] = latent[4];
