@@ -29,41 +29,12 @@
 #include "md5.h"
 #include "freedom.h"
 
-int
-scan_subject(const char *bodyfile)
-{
-  int found_subject = 0;
-  FILE *in;
-  char line[BUFSIZ], tmpline[BUFSIZ];
-
-  in = fopen(bodyfile, "r");
-  while (fgets(line, sizeof(line), in)) {
-    if (strlen(line) <= 1) {
-      fclose(in);
-      return found_subject;
-    }
-    else {
-      if (strileft(line, "subject:")) {
-	found_subject++;
-	strcpy (tmpline, line + sizeof("subject:") - 1);
-	chop(tmpline);
-	if (blocked(tmpline, SUBJ_BLOCK)) {
-	  fclose(in);
-	  return -1;
-	}
-      }
-    }
-  }
-  /* no blank line found */
-  fclose(in);
-  return found_subject;
-}
-
 void
 encrypt_sub(const char *origfilename, const char *key)
 /* IDEA-encrypt MD5-hash of ##'ed Subject: line with MD5-hash of "key" */
 {
   FILE *infile, *outfile;
+  long offset;
   int headers = 1, colhead = 0, addhead = 0, linenum = 0, x;
   char tempfilename[BUFSIZ], line[BUFSIZ], tmpline[BUFSIZ];
   struct MD5Context subjkeyDigest, subjectDigest;
@@ -88,19 +59,28 @@ encrypt_sub(const char *origfilename, const char *key)
 
   while (fgets(line, sizeof(line), infile)) {
     if (!headers) {
-      if ((linenum > colhead) || (strlen(line) > 1))
+      if (linenum > colhead || strlen(line) > 1)
 	linenum++;
-      if ((linenum == 1) && strstr(line, "::")) {
+      if (linenum == 1 && !strncmp(line, "::", 2)) {
 	headers = 1;
 	colhead = 1;
 	fputs(line, outfile);
       }
-      else if ((linenum == (colhead + 1)) && strstr(line, "##")) {
+      else if (linenum == colhead + 1 && !strncmp(line, "##", 2)) {
 	addhead = 1;
 	fputs(line, outfile);
       }
       else {
-	if (addhead) {
+	if (addhead) { /* read multi-line headers */
+	  for (;;) {
+	  offset = ftell(infile);
+	    if (fgets(tmpline, sizeof(tmpline), infile) &&
+		(tmpline[0] == ' ' || tmpline[0] == '\t'))
+	      strncat(line, tmpline, sizeof(line) - strlen(line) - 1);
+	    else
+	      break;
+	  }
+	  fseek(infile, offset, SEEK_SET);
 	  if (strlen(line) <= 1)
 	    addhead = 0;
 	  if (!strileft(line, "subject:"))
@@ -108,10 +88,10 @@ encrypt_sub(const char *origfilename, const char *key)
 	  else {
 	    if (USE_SYSLOG)
 	      syslog(LOG_DEBUG, "Encrypting %s", line);
-	    strcpy(tmpline, line + sizeof("subject:") - 1);
-	    chop(tmpline);
+	    flushleft(line, sizeof("subject:") - 1);
+	    chop(line);
 	    MD5Init(&subjectDigest);
-	    MD5Update(&subjectDigest, tmpline, strlen(tmpline));
+	    MD5Update(&subjectDigest, line, strlen(line));
 	    MD5Final(subjPT, &subjectDigest);
 	    ideaCfbInit(&subjcontext, subjIDEAkey);
 	    sprintf(hexrand, "%08x", rand());
@@ -119,23 +99,23 @@ encrypt_sub(const char *origfilename, const char *key)
 	    memcpy(subjCT + 8, subjPT, 16) ;
 	    ideaCfbEncrypt(&subjcontext, subjCT, subjCT, 24);
 	    ideaCfbDestroy(&subjcontext);
-	    fprintf(outfile, "Subject: ");
+	    fputs("Subject: ", outfile);
 	    for (x = 0; x < 24; x++)
 	      fprintf(outfile, "%2.2hx", subjCT[x]);
-	    fprintf(outfile, "\n");
+	    fputs("\n", outfile);
 	  }
 	}
 	else {
-	  if ((linenum > colhead) || (strlen(line) > 1))
+	  if (linenum > colhead || strlen(line) > 1)
 	    fputs(line, outfile);
 	}
       }
     }
     else {
-      if (strlen(line) <= 1)
-	headers = 0;
       if (!strileft(line, "encrypt-subject:"))
 	fputs(line, outfile);
+      if (strlen(line) <= 1)
+	headers = 0;
     }
   }
   fclose(infile);
